/* Events: Sample, Zero Crossings, Relations, Discrete Changes */
#include "BouncingBall_model.h"
#if defined(__cplusplus)
extern "C" {
#endif

/* Initializes the raw time events of the simulation using the now
   calcualted parameters. */
void BouncingBall_function_initSample(DATA *data, threadData_t *threadData)
{
  long i=0;
}

const char *BouncingBall_zeroCrossingDescription(int i, int **out_EquationIndexes)
{
  static const char *res[] = {"h <= 0.0 and v <= 0.0",
  "h <= 0.0",
  "v <= 0.0"};
  static const int occurEqs0[] = {1,22};
  static const int occurEqs1[] = {1,14};
  static const int occurEqs2[] = {1,16};
  static const int *occurEqs[] = {occurEqs0,occurEqs1,occurEqs2};
  *out_EquationIndexes = (int*) occurEqs[i];
  return res[i];
}

/* forwarded equations */
extern void BouncingBall_eqFunction_14(DATA* data, threadData_t *threadData);
extern void BouncingBall_eqFunction_16(DATA* data, threadData_t *threadData);
extern void BouncingBall_eqFunction_17(DATA* data, threadData_t *threadData);
extern void BouncingBall_eqFunction_20(DATA* data, threadData_t *threadData);
extern void BouncingBall_eqFunction_21(DATA* data, threadData_t *threadData);

int BouncingBall_function_ZeroCrossingsEquations(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH

  data->simulationInfo->callStatistics.functionZeroCrossingsEquations++;

  BouncingBall_eqFunction_14(data, threadData);

  BouncingBall_eqFunction_16(data, threadData);

  BouncingBall_eqFunction_17(data, threadData);

  BouncingBall_eqFunction_20(data, threadData);

  BouncingBall_eqFunction_21(data, threadData);
  
  TRACE_POP
  return 0;
}

int BouncingBall_function_ZeroCrossings(DATA *data, threadData_t *threadData, double *gout)
{
  TRACE_PUSH
  const int *equationIndexes = NULL;

  modelica_boolean tmp0;
  modelica_real tmp1;
  modelica_real tmp2;
  modelica_boolean tmp3;
  modelica_real tmp4;
  modelica_real tmp5;
  modelica_boolean tmp6;
  modelica_real tmp7;
  modelica_real tmp8;
  modelica_boolean tmp9;
  modelica_real tmp10;
  modelica_real tmp11;

#if !defined(OMC_MINIMAL_RUNTIME)
  if (measure_time_flag) rt_tick(SIM_TIMER_ZC);
#endif
  data->simulationInfo->callStatistics.functionZeroCrossings++;

  tmp1 = 1.0;
  tmp2 = 0.0;
  tmp0 = LessEqZC((data->localData[0]->realVars[0] /* h STATE(1,v) */), 0.0, tmp1, tmp2, data->simulationInfo->storedRelations[0]);
  tmp4 = 1.0;
  tmp5 = 0.0;
  tmp3 = LessEqZC((data->localData[0]->realVars[1] /* v STATE(1) */), 0.0, tmp4, tmp5, data->simulationInfo->storedRelations[1]);
  gout[0] = ((tmp0 && tmp3)) ? 1 : -1;

  tmp7 = 1.0;
  tmp8 = 0.0;
  tmp6 = LessEqZC((data->localData[0]->realVars[0] /* h STATE(1,v) */), 0.0, tmp7, tmp8, data->simulationInfo->storedRelations[0]);
  gout[1] = (tmp6) ? 1 : -1;

  tmp10 = 1.0;
  tmp11 = 0.0;
  tmp9 = LessEqZC((data->localData[0]->realVars[1] /* v STATE(1) */), 0.0, tmp10, tmp11, data->simulationInfo->storedRelations[1]);
  gout[2] = (tmp9) ? 1 : -1;

#if !defined(OMC_MINIMAL_RUNTIME)
  if (measure_time_flag) rt_accumulate(SIM_TIMER_ZC);
#endif

  TRACE_POP
  return 0;
}

const char *BouncingBall_relationDescription(int i)
{
  const char *res[] = {"h <= 0.0",
  "v <= 0.0"};
  return res[i];
}

int BouncingBall_function_updateRelations(DATA *data, threadData_t *threadData, int evalforZeroCross)
{
  TRACE_PUSH
  const int *equationIndexes = NULL;

  modelica_boolean tmp12;
  modelica_real tmp13;
  modelica_real tmp14;
  modelica_boolean tmp15;
  modelica_real tmp16;
  modelica_real tmp17;
  
  if(evalforZeroCross) {
    tmp13 = 1.0;
    tmp14 = 0.0;
    tmp12 = LessEqZC((data->localData[0]->realVars[0] /* h STATE(1,v) */), 0.0, tmp13, tmp14, data->simulationInfo->storedRelations[0]);
    data->simulationInfo->relations[0] = tmp12;

    tmp16 = 1.0;
    tmp17 = 0.0;
    tmp15 = LessEqZC((data->localData[0]->realVars[1] /* v STATE(1) */), 0.0, tmp16, tmp17, data->simulationInfo->storedRelations[1]);
    data->simulationInfo->relations[1] = tmp15;
  } else {
    data->simulationInfo->relations[0] = ((data->localData[0]->realVars[0] /* h STATE(1,v) */) <= 0.0);

    data->simulationInfo->relations[1] = ((data->localData[0]->realVars[1] /* v STATE(1) */) <= 0.0);
  }
  
  TRACE_POP
  return 0;
}

#if defined(__cplusplus)
}
#endif

